"""
Type annotations for geo-routes service literal definitions.

[Documentation](https://youtype.github.io/types_boto3_docs/types_boto3_geo_routes/literals/)

Copyright 2025 Vlad Emelianov

Usage::

    ```python
    from types_boto3_geo_routes.literals import DayOfWeekType

    data: DayOfWeekType = "Friday"
    ```
"""

import sys

if sys.version_info >= (3, 12):
    from typing import Literal
else:
    from typing_extensions import Literal

__all__ = (
    "DayOfWeekType",
    "GeometryFormatType",
    "IsolineEngineTypeType",
    "IsolineHazardousCargoTypeType",
    "IsolineOptimizationObjectiveType",
    "IsolineTravelModeType",
    "IsolineTruckTypeType",
    "IsolineZoneCategoryType",
    "LocationServiceRoutesV2ServiceName",
    "MatchingStrategyType",
    "MeasurementSystemType",
    "ResourceServiceName",
    "RoadSnapHazardousCargoTypeType",
    "RoadSnapNoticeCodeType",
    "RoadSnapTravelModeType",
    "RouteDirectionType",
    "RouteEngineTypeType",
    "RouteFerryAfterTravelStepTypeType",
    "RouteFerryBeforeTravelStepTypeType",
    "RouteFerryNoticeCodeType",
    "RouteFerryTravelStepTypeType",
    "RouteHazardousCargoTypeType",
    "RouteLegAdditionalFeatureType",
    "RouteLegTravelModeType",
    "RouteLegTypeType",
    "RouteMatrixErrorCodeType",
    "RouteMatrixHazardousCargoTypeType",
    "RouteMatrixTravelModeType",
    "RouteMatrixTruckTypeType",
    "RouteMatrixZoneCategoryType",
    "RouteNoticeImpactType",
    "RoutePedestrianNoticeCodeType",
    "RoutePedestrianTravelStepTypeType",
    "RouteResponseNoticeCodeType",
    "RouteRoadTypeType",
    "RouteSideOfStreetType",
    "RouteSpanAdditionalFeatureType",
    "RouteSpanCarAccessAttributeType",
    "RouteSpanGateAttributeType",
    "RouteSpanPedestrianAccessAttributeType",
    "RouteSpanRailwayCrossingAttributeType",
    "RouteSpanRoadAttributeType",
    "RouteSpanScooterAccessAttributeType",
    "RouteSpanTruckAccessAttributeType",
    "RouteSteeringDirectionType",
    "RouteTollPassValidityPeriodTypeType",
    "RouteTollPaymentMethodType",
    "RouteTollVehicleCategoryType",
    "RouteTravelModeType",
    "RouteTravelStepTypeType",
    "RouteTruckTypeType",
    "RouteTurnIntensityType",
    "RouteVehicleIncidentSeverityType",
    "RouteVehicleIncidentTypeType",
    "RouteVehicleNoticeCodeType",
    "RouteVehicleTravelStepTypeType",
    "RouteWeightConstraintTypeType",
    "RouteZoneCategoryType",
    "RoutingObjectiveType",
    "ServiceName",
    "SideOfStreetMatchingStrategyType",
    "TrafficUsageType",
    "WaypointOptimizationClusteringAlgorithmType",
    "WaypointOptimizationConstraintType",
    "WaypointOptimizationHazardousCargoTypeType",
    "WaypointOptimizationSequencingObjectiveType",
    "WaypointOptimizationServiceTimeTreatmentType",
    "WaypointOptimizationTravelModeType",
    "WaypointOptimizationTruckTypeType",
)

DayOfWeekType = Literal[
    "Friday", "Monday", "Saturday", "Sunday", "Thursday", "Tuesday", "Wednesday"
]
GeometryFormatType = Literal["FlexiblePolyline", "Simple"]
IsolineEngineTypeType = Literal["Electric", "InternalCombustion", "PluginHybrid"]
IsolineHazardousCargoTypeType = Literal[
    "Combustible",
    "Corrosive",
    "Explosive",
    "Flammable",
    "Gas",
    "HarmfulToWater",
    "Organic",
    "Other",
    "Poison",
    "PoisonousInhalation",
    "Radioactive",
]
IsolineOptimizationObjectiveType = Literal[
    "AccurateCalculation", "BalancedCalculation", "FastCalculation"
]
IsolineTravelModeType = Literal["Car", "Pedestrian", "Scooter", "Truck"]
IsolineTruckTypeType = Literal["LightTruck", "StraightTruck", "Tractor"]
IsolineZoneCategoryType = Literal["CongestionPricing", "Environmental", "Vignette"]
MatchingStrategyType = Literal["MatchAny", "MatchMostSignificantRoad"]
MeasurementSystemType = Literal["Imperial", "Metric"]
RoadSnapHazardousCargoTypeType = Literal[
    "Combustible",
    "Corrosive",
    "Explosive",
    "Flammable",
    "Gas",
    "HarmfulToWater",
    "Organic",
    "Other",
    "Poison",
    "PoisonousInhalation",
    "Radioactive",
]
RoadSnapNoticeCodeType = Literal[
    "TracePointsHeadingIgnored",
    "TracePointsIgnored",
    "TracePointsMovedByLargeDistance",
    "TracePointsNotMatched",
    "TracePointsOutOfSequence",
    "TracePointsSpeedEstimated",
    "TracePointsSpeedIgnored",
]
RoadSnapTravelModeType = Literal["Car", "Pedestrian", "Scooter", "Truck"]
RouteDirectionType = Literal["East", "North", "South", "West"]
RouteEngineTypeType = Literal["Electric", "InternalCombustion", "PluginHybrid"]
RouteFerryAfterTravelStepTypeType = Literal["Deboard"]
RouteFerryBeforeTravelStepTypeType = Literal["Board"]
RouteFerryNoticeCodeType = Literal[
    "AccuratePolylineUnavailable",
    "NoSchedule",
    "Other",
    "PotentialViolatedVehicleRestrictionUsage",
    "SeasonalClosure",
    "ViolatedAvoidFerry",
    "ViolatedAvoidRailFerry",
]
RouteFerryTravelStepTypeType = Literal["Arrive", "Continue", "Depart"]
RouteHazardousCargoTypeType = Literal[
    "Combustible",
    "Corrosive",
    "Explosive",
    "Flammable",
    "Gas",
    "HarmfulToWater",
    "Organic",
    "Other",
    "Poison",
    "PoisonousInhalation",
    "Radioactive",
]
RouteLegAdditionalFeatureType = Literal[
    "Elevation",
    "Incidents",
    "PassThroughWaypoints",
    "Summary",
    "Tolls",
    "TravelStepInstructions",
    "TruckRoadTypes",
    "TypicalDuration",
    "Zones",
]
RouteLegTravelModeType = Literal[
    "Car", "CarShuttleTrain", "Ferry", "Pedestrian", "Scooter", "Truck"
]
RouteLegTypeType = Literal["Ferry", "Pedestrian", "Vehicle"]
RouteMatrixErrorCodeType = Literal[
    "NoMatch",
    "NoMatchDestination",
    "NoMatchOrigin",
    "NoRoute",
    "Other",
    "OutOfBounds",
    "OutOfBoundsDestination",
    "OutOfBoundsOrigin",
    "Violation",
]
RouteMatrixHazardousCargoTypeType = Literal[
    "Combustible",
    "Corrosive",
    "Explosive",
    "Flammable",
    "Gas",
    "HarmfulToWater",
    "Organic",
    "Other",
    "Poison",
    "PoisonousInhalation",
    "Radioactive",
]
RouteMatrixTravelModeType = Literal["Car", "Pedestrian", "Scooter", "Truck"]
RouteMatrixTruckTypeType = Literal["LightTruck", "StraightTruck", "Tractor"]
RouteMatrixZoneCategoryType = Literal["CongestionPricing", "Environmental", "Vignette"]
RouteNoticeImpactType = Literal["High", "Low"]
RoutePedestrianNoticeCodeType = Literal[
    "AccuratePolylineUnavailable",
    "Other",
    "ViolatedAvoidDirtRoad",
    "ViolatedAvoidTunnel",
    "ViolatedPedestrianOption",
]
RoutePedestrianTravelStepTypeType = Literal[
    "Arrive",
    "Continue",
    "Depart",
    "Exit",
    "Keep",
    "Ramp",
    "RoundaboutEnter",
    "RoundaboutExit",
    "RoundaboutPass",
    "Turn",
    "UTurn",
]
RouteResponseNoticeCodeType = Literal[
    "MainLanguageNotFound", "Other", "TravelTimeExceedsDriverWorkHours"
]
RouteRoadTypeType = Literal["Highway", "Rural", "Urban"]
RouteSideOfStreetType = Literal["Left", "Right"]
RouteSpanAdditionalFeatureType = Literal[
    "BestCaseDuration",
    "CarAccess",
    "Consumption",
    "Country",
    "Distance",
    "Duration",
    "DynamicSpeed",
    "FunctionalClassification",
    "Gates",
    "Incidents",
    "Names",
    "Notices",
    "PedestrianAccess",
    "RailwayCrossings",
    "Region",
    "RoadAttributes",
    "RouteNumbers",
    "ScooterAccess",
    "SpeedLimit",
    "TollSystems",
    "TruckAccess",
    "TruckRoadTypes",
    "TypicalDuration",
    "Zones",
]
RouteSpanCarAccessAttributeType = Literal["Allowed", "NoThroughTraffic", "TollRoad"]
RouteSpanGateAttributeType = Literal["Emergency", "KeyAccess", "PermissionRequired"]
RouteSpanPedestrianAccessAttributeType = Literal[
    "Allowed", "Indoors", "NoThroughTraffic", "Park", "Stairs", "TollRoad"
]
RouteSpanRailwayCrossingAttributeType = Literal["Protected", "Unprotected"]
RouteSpanRoadAttributeType = Literal[
    "Bridge",
    "BuiltUpArea",
    "ControlledAccessHighway",
    "DirtRoad",
    "DividedRoad",
    "Motorway",
    "PrivateRoad",
    "Ramp",
    "RightHandTraffic",
    "Roundabout",
    "Tunnel",
    "UnderConstruction",
]
RouteSpanScooterAccessAttributeType = Literal["Allowed", "NoThroughTraffic", "TollRoad"]
RouteSpanTruckAccessAttributeType = Literal["Allowed", "NoThroughTraffic", "TollRoad"]
RouteSteeringDirectionType = Literal["Left", "Right", "Straight"]
RouteTollPassValidityPeriodTypeType = Literal[
    "Annual", "Days", "ExtendedAnnual", "Minutes", "Months"
]
RouteTollPaymentMethodType = Literal[
    "BankCard",
    "Cash",
    "CashExact",
    "CreditCard",
    "PassSubscription",
    "Transponder",
    "TravelCard",
    "VideoToll",
]
RouteTollVehicleCategoryType = Literal["Minibus"]
RouteTravelModeType = Literal["Car", "Pedestrian", "Scooter", "Truck"]
RouteTravelStepTypeType = Literal["Default", "TurnByTurn"]
RouteTruckTypeType = Literal["LightTruck", "StraightTruck", "Tractor"]
RouteTurnIntensityType = Literal["Sharp", "Slight", "Typical"]
RouteVehicleIncidentSeverityType = Literal["Critical", "High", "Low", "Medium"]
RouteVehicleIncidentTypeType = Literal[
    "Accident",
    "Congestion",
    "Construction",
    "DisabledVehicle",
    "LaneRestriction",
    "MassTransit",
    "Other",
    "PlannedEvent",
    "RoadClosure",
    "RoadHazard",
    "Weather",
]
RouteVehicleNoticeCodeType = Literal[
    "AccuratePolylineUnavailable",
    "Other",
    "PotentialViolatedAvoidTollRoadUsage",
    "PotentialViolatedCarpoolUsage",
    "PotentialViolatedTurnRestrictionUsage",
    "PotentialViolatedVehicleRestrictionUsage",
    "PotentialViolatedZoneRestrictionUsage",
    "SeasonalClosure",
    "TollTransponder",
    "TollsDataTemporarilyUnavailable",
    "TollsDataUnavailable",
    "ViolatedAvoidControlledAccessHighway",
    "ViolatedAvoidDifficultTurns",
    "ViolatedAvoidDirtRoad",
    "ViolatedAvoidSeasonalClosure",
    "ViolatedAvoidTollRoad",
    "ViolatedAvoidTollTransponder",
    "ViolatedAvoidTruckRoadType",
    "ViolatedAvoidTunnel",
    "ViolatedAvoidUTurns",
    "ViolatedBlockedRoad",
    "ViolatedCarpool",
    "ViolatedEmergencyGate",
    "ViolatedStartDirection",
    "ViolatedTurnRestriction",
    "ViolatedVehicleRestriction",
    "ViolatedZoneRestriction",
]
RouteVehicleTravelStepTypeType = Literal[
    "Arrive",
    "Continue",
    "ContinueHighway",
    "Depart",
    "EnterHighway",
    "Exit",
    "Keep",
    "Ramp",
    "RoundaboutEnter",
    "RoundaboutExit",
    "RoundaboutPass",
    "Turn",
    "UTurn",
]
RouteWeightConstraintTypeType = Literal["Current", "Gross", "Unknown"]
RouteZoneCategoryType = Literal["CongestionPricing", "Environmental", "Vignette"]
RoutingObjectiveType = Literal["FastestRoute", "ShortestRoute"]
SideOfStreetMatchingStrategyType = Literal["AnyStreet", "DividedStreetOnly"]
TrafficUsageType = Literal["IgnoreTrafficData", "UseTrafficData"]
WaypointOptimizationClusteringAlgorithmType = Literal["DrivingDistance", "TopologySegment"]
WaypointOptimizationConstraintType = Literal[
    "AccessHours", "AppointmentTime", "Before", "Heading", "ServiceDuration", "SideOfStreet"
]
WaypointOptimizationHazardousCargoTypeType = Literal[
    "Combustible",
    "Corrosive",
    "Explosive",
    "Flammable",
    "Gas",
    "HarmfulToWater",
    "Organic",
    "Other",
    "Poison",
    "PoisonousInhalation",
    "Radioactive",
]
WaypointOptimizationSequencingObjectiveType = Literal["FastestRoute", "ShortestRoute"]
WaypointOptimizationServiceTimeTreatmentType = Literal["Rest", "Work"]
WaypointOptimizationTravelModeType = Literal["Car", "Pedestrian", "Scooter", "Truck"]
WaypointOptimizationTruckTypeType = Literal["StraightTruck", "Tractor"]
LocationServiceRoutesV2ServiceName = Literal["geo-routes"]
ServiceName = Literal[
    "accessanalyzer",
    "account",
    "acm",
    "acm-pca",
    "aiops",
    "amp",
    "amplify",
    "amplifybackend",
    "amplifyuibuilder",
    "apigateway",
    "apigatewaymanagementapi",
    "apigatewayv2",
    "appconfig",
    "appconfigdata",
    "appfabric",
    "appflow",
    "appintegrations",
    "application-autoscaling",
    "application-insights",
    "application-signals",
    "applicationcostprofiler",
    "appmesh",
    "apprunner",
    "appstream",
    "appsync",
    "arc-region-switch",
    "arc-zonal-shift",
    "artifact",
    "athena",
    "auditmanager",
    "autoscaling",
    "autoscaling-plans",
    "b2bi",
    "backup",
    "backup-gateway",
    "backupsearch",
    "batch",
    "bcm-dashboards",
    "bcm-data-exports",
    "bcm-pricing-calculator",
    "bcm-recommended-actions",
    "bedrock",
    "bedrock-agent",
    "bedrock-agent-runtime",
    "bedrock-agentcore",
    "bedrock-agentcore-control",
    "bedrock-data-automation",
    "bedrock-data-automation-runtime",
    "bedrock-runtime",
    "billing",
    "billingconductor",
    "braket",
    "budgets",
    "ce",
    "chatbot",
    "chime",
    "chime-sdk-identity",
    "chime-sdk-media-pipelines",
    "chime-sdk-meetings",
    "chime-sdk-messaging",
    "chime-sdk-voice",
    "cleanrooms",
    "cleanroomsml",
    "cloud9",
    "cloudcontrol",
    "clouddirectory",
    "cloudformation",
    "cloudfront",
    "cloudfront-keyvaluestore",
    "cloudhsm",
    "cloudhsmv2",
    "cloudsearch",
    "cloudsearchdomain",
    "cloudtrail",
    "cloudtrail-data",
    "cloudwatch",
    "codeartifact",
    "codebuild",
    "codecatalyst",
    "codecommit",
    "codeconnections",
    "codedeploy",
    "codeguru-reviewer",
    "codeguru-security",
    "codeguruprofiler",
    "codepipeline",
    "codestar-connections",
    "codestar-notifications",
    "cognito-identity",
    "cognito-idp",
    "cognito-sync",
    "comprehend",
    "comprehendmedical",
    "compute-optimizer",
    "config",
    "connect",
    "connect-contact-lens",
    "connectcampaigns",
    "connectcampaignsv2",
    "connectcases",
    "connectparticipant",
    "controlcatalog",
    "controltower",
    "cost-optimization-hub",
    "cur",
    "customer-profiles",
    "databrew",
    "dataexchange",
    "datapipeline",
    "datasync",
    "datazone",
    "dax",
    "deadline",
    "detective",
    "devicefarm",
    "devops-guru",
    "directconnect",
    "discovery",
    "dlm",
    "dms",
    "docdb",
    "docdb-elastic",
    "drs",
    "ds",
    "ds-data",
    "dsql",
    "dynamodb",
    "dynamodbstreams",
    "ebs",
    "ec2",
    "ec2-instance-connect",
    "ecr",
    "ecr-public",
    "ecs",
    "efs",
    "eks",
    "eks-auth",
    "elasticache",
    "elasticbeanstalk",
    "elastictranscoder",
    "elb",
    "elbv2",
    "emr",
    "emr-containers",
    "emr-serverless",
    "entityresolution",
    "es",
    "events",
    "evidently",
    "evs",
    "finspace",
    "finspace-data",
    "firehose",
    "fis",
    "fms",
    "forecast",
    "forecastquery",
    "frauddetector",
    "freetier",
    "fsx",
    "gamelift",
    "gameliftstreams",
    "geo-maps",
    "geo-places",
    "geo-routes",
    "glacier",
    "globalaccelerator",
    "glue",
    "grafana",
    "greengrass",
    "greengrassv2",
    "groundstation",
    "guardduty",
    "health",
    "healthlake",
    "iam",
    "identitystore",
    "imagebuilder",
    "importexport",
    "inspector",
    "inspector-scan",
    "inspector2",
    "internetmonitor",
    "invoicing",
    "iot",
    "iot-data",
    "iot-jobs-data",
    "iot-managed-integrations",
    "iotanalytics",
    "iotdeviceadvisor",
    "iotevents",
    "iotevents-data",
    "iotfleetwise",
    "iotsecuretunneling",
    "iotsitewise",
    "iotthingsgraph",
    "iottwinmaker",
    "iotwireless",
    "ivs",
    "ivs-realtime",
    "ivschat",
    "kafka",
    "kafkaconnect",
    "kendra",
    "kendra-ranking",
    "keyspaces",
    "keyspacesstreams",
    "kinesis",
    "kinesis-video-archived-media",
    "kinesis-video-media",
    "kinesis-video-signaling",
    "kinesis-video-webrtc-storage",
    "kinesisanalytics",
    "kinesisanalyticsv2",
    "kinesisvideo",
    "kms",
    "lakeformation",
    "lambda",
    "launch-wizard",
    "lex-models",
    "lex-runtime",
    "lexv2-models",
    "lexv2-runtime",
    "license-manager",
    "license-manager-linux-subscriptions",
    "license-manager-user-subscriptions",
    "lightsail",
    "location",
    "logs",
    "lookoutequipment",
    "m2",
    "machinelearning",
    "macie2",
    "mailmanager",
    "managedblockchain",
    "managedblockchain-query",
    "marketplace-agreement",
    "marketplace-catalog",
    "marketplace-deployment",
    "marketplace-entitlement",
    "marketplace-reporting",
    "marketplacecommerceanalytics",
    "mediaconnect",
    "mediaconvert",
    "medialive",
    "mediapackage",
    "mediapackage-vod",
    "mediapackagev2",
    "mediastore",
    "mediastore-data",
    "mediatailor",
    "medical-imaging",
    "memorydb",
    "meteringmarketplace",
    "mgh",
    "mgn",
    "migration-hub-refactor-spaces",
    "migrationhub-config",
    "migrationhuborchestrator",
    "migrationhubstrategy",
    "mpa",
    "mq",
    "mturk",
    "mwaa",
    "mwaa-serverless",
    "neptune",
    "neptune-graph",
    "neptunedata",
    "network-firewall",
    "networkflowmonitor",
    "networkmanager",
    "networkmonitor",
    "notifications",
    "notificationscontacts",
    "oam",
    "observabilityadmin",
    "odb",
    "omics",
    "opensearch",
    "opensearchserverless",
    "organizations",
    "osis",
    "outposts",
    "panorama",
    "partnercentral-channel",
    "partnercentral-selling",
    "payment-cryptography",
    "payment-cryptography-data",
    "pca-connector-ad",
    "pca-connector-scep",
    "pcs",
    "personalize",
    "personalize-events",
    "personalize-runtime",
    "pi",
    "pinpoint",
    "pinpoint-email",
    "pinpoint-sms-voice",
    "pinpoint-sms-voice-v2",
    "pipes",
    "polly",
    "pricing",
    "proton",
    "qapps",
    "qbusiness",
    "qconnect",
    "quicksight",
    "ram",
    "rbin",
    "rds",
    "rds-data",
    "redshift",
    "redshift-data",
    "redshift-serverless",
    "rekognition",
    "repostspace",
    "resiliencehub",
    "resource-explorer-2",
    "resource-groups",
    "resourcegroupstaggingapi",
    "rolesanywhere",
    "route53",
    "route53-recovery-cluster",
    "route53-recovery-control-config",
    "route53-recovery-readiness",
    "route53domains",
    "route53profiles",
    "route53resolver",
    "rtbfabric",
    "rum",
    "s3",
    "s3control",
    "s3outposts",
    "s3tables",
    "s3vectors",
    "sagemaker",
    "sagemaker-a2i-runtime",
    "sagemaker-edge",
    "sagemaker-featurestore-runtime",
    "sagemaker-geospatial",
    "sagemaker-metrics",
    "sagemaker-runtime",
    "savingsplans",
    "scheduler",
    "schemas",
    "sdb",
    "secretsmanager",
    "security-ir",
    "securityhub",
    "securitylake",
    "serverlessrepo",
    "service-quotas",
    "servicecatalog",
    "servicecatalog-appregistry",
    "servicediscovery",
    "ses",
    "sesv2",
    "shield",
    "signer",
    "signin",
    "simspaceweaver",
    "snow-device-management",
    "snowball",
    "sns",
    "socialmessaging",
    "sqs",
    "ssm",
    "ssm-contacts",
    "ssm-guiconnect",
    "ssm-incidents",
    "ssm-quicksetup",
    "ssm-sap",
    "sso",
    "sso-admin",
    "sso-oidc",
    "stepfunctions",
    "storagegateway",
    "sts",
    "supplychain",
    "support",
    "support-app",
    "swf",
    "synthetics",
    "taxsettings",
    "textract",
    "timestream-influxdb",
    "timestream-query",
    "timestream-write",
    "tnb",
    "transcribe",
    "transfer",
    "translate",
    "trustedadvisor",
    "verifiedpermissions",
    "voice-id",
    "vpc-lattice",
    "waf",
    "waf-regional",
    "wafv2",
    "wellarchitected",
    "wisdom",
    "workdocs",
    "workmail",
    "workmailmessageflow",
    "workspaces",
    "workspaces-instances",
    "workspaces-thin-client",
    "workspaces-web",
    "xray",
]
ResourceServiceName = Literal[
    "cloudformation", "cloudwatch", "dynamodb", "ec2", "glacier", "iam", "s3", "sns", "sqs"
]
